@@ -3,8 +3,6 @@
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-from __future__ import print_function
-
 import contextlib
 from distutils.version import LooseVersion
 import errno
@@ -19,9 +17,11 @@ import shutil
 import site
 import sys
 import tarfile
+from tempfile import TemporaryDirectory
 from threading import Event
+from urllib.request import Request, urlopen, urljoin, quote
+from urllib.error import URLError
 
-from ipython_genutils.tempdir import TemporaryDirectory
 from jupyter_core.paths import jupyter_config_path
 from jupyterlab_launcher.process import which, Process, WatchHelper
 from notebook.nbextensions import GREEN_ENABLED, GREEN_OK, RED_DISABLED, RED_X
@@ -29,14 +29,6 @@ from notebook.nbextensions import GREEN_ENABLED, GREEN_OK, RED_DISABLED, RED_X
 from .semver import Range, gte, lt, lte, gt, make_semver
 from .jlpmapp import YARN_PATH, HERE
 
-if sys.version_info.major < 3:
-    from urllib2 import Request, urlopen, quote
-    from urllib2 import URLError, HTTPError
-    from urlparse import urljoin
-else:
-    from urllib.request import Request, urlopen, urljoin, quote
-    from urllib.error import URLError, HTTPError
-
 
 # The regex for expecting the webpack output.
 WEBPACK_EXPECT = re.compile(r'.*/index.out.js')
@@ -883,7 +875,15 @@ class _AppHandler(object):
         templates = pjoin(staging, 'templates')
         if osp.exists(templates):
             shutil.rmtree(templates)
-        shutil.copytree(pjoin(HERE, 'staging', 'templates'), templates)
+
+        try:
+            shutil.copytree(pjoin(HERE, 'staging', 'templates'), templates)
+        except shutil.Error as error:
+            # `copytree` throws an error if copying to + from NFS even though
+            # the copy is successful (see https://bugs.python.org/issue24564)
+
+            if '[Errno 22]' not in str(error) or not osp.exists(templates):
+                raise
 
         # Ensure a clean linked packages directory.
         linked_dir = pjoin(staging, 'linked_packages')
@@ -971,11 +971,10 @@ class _AppHandler(object):
             # Reject incompatible extensions with a message.
             errors = compat_errors[key]
             if errors:
-                msg = _format_compatibility_errors(
-                    key, value['version'], errors
-                )
                 if not silent:
-                    logger.warn(msg + '\n')
+                    _log_single_compat_errors(
+                        logger, key, value['version'], errors
+                    )
                 continue
 
             data['dependencies'][key] = format_path(value['path'])
@@ -1069,8 +1068,16 @@ class _AppHandler(object):
             name = data['name']
             jlab = data.get('jupyterlab', dict())
             path = osp.realpath(target)
+            # homepage, repository  are optional
+            if 'homepage' in data:
+                url = data['homepage']
+            elif 'repository' in data and isinstance(data['repository'], dict):
+                url = data['repository'].get('url', '')
+            else:
+                url = ''
             extensions[name] = dict(path=path,
                                     filename=osp.basename(path),
+                                    url=url,
                                     version=data['version'],
                                     jupyterlab=jlab,
                                     dependencies=deps,
@@ -1147,9 +1154,10 @@ class _AppHandler(object):
 
         dname = info['%s_dir' % ext_type]
 
+        error_accumulator = {}
+
         logger.info('   %s dir: %s' % (ext_type, dname))
         for name in sorted(names):
-            logger.info(name)
             data = info['extensions'][name]
             version = data['version']
             errors = info['compat_errors'][name]
@@ -1166,10 +1174,10 @@ class _AppHandler(object):
                 extra += '*'
             logger.info('        %s v%s%s' % (name, version, extra))
             if errors:
-                msg = _format_compatibility_errors(
-                    name, version, errors
-                )
-                logger.warn(msg + '\n')
+                error_accumulator[name] = (version, errors)
+
+        # Write all errors at end:
+        _log_multiple_compat_errors(logger, error_accumulator)
 
     def _read_build_config(self):
         """Get the build config data for the app dir.
@@ -1463,9 +1471,10 @@ class _AppHandler(object):
 def _node_check():
     """Check for the existence of nodejs with the correct version.
     """
-    proc = Process(['node', 'node-version-check.js'], cwd=HERE, quiet=True)
-    return_code = proc.wait()
-    if return_code == 1:
+    try:
+        proc = Process(['node', 'node-version-check.js'], cwd=HERE, quiet=True)
+        proc.wait()
+    except Exception:
         msg = 'Please install nodejs 5+ and npm before continuing. nodejs may be installed using conda or directly from the nodejs website.'
         raise ValueError(msg)
 
@@ -1693,6 +1702,67 @@ def _format_compatibility_errors(name, version, errors):
     return msg
 
 
+def _log_multiple_compat_errors(logger, errors_map):
+    """Log compatability errors for multiple extensions at once"""
+
+    outdated = []
+    others = []
+
+    for name, (version, errors) in errors_map.items():
+        age = _compat_error_age(errors)
+        if age > 0:
+            outdated.append(name)
+        else:
+            others.append(name)
+
+    if outdated:
+        logger.warn('\n        '.join(
+            ['\n   The following extension are outdated:'] +
+            outdated +
+            ['\n   Consider running "jupyter labextension update --all" '
+             'to check for updates.\n']
+        ))
+
+    for name in others:
+        version, errors = errors_map[name]
+        msg = _format_compatibility_errors(name, version, errors)
+        logger.warn(msg + '\n')
+
+
+def _log_single_compat_errors(logger, name, version, errors):
+    """Log compatability errors for a single extension"""
+
+    age = _compat_error_age(errors)
+    if age > 0:
+        logger.warn('The extension "%s" is outdated.\n', name)
+    else:
+        msg = _format_compatibility_errors(name, version, errors)
+        logger.warn(msg + '\n')
+
+
+def _compat_error_age(errors):
+    """Compare all incompatabilites for an extension.
+
+    Returns a number > 0 if all extensions are older than that supported by lab.
+    Returns a number < 0 if all extensions are newer than that supported by lab.
+    Returns 0 otherwise (i.e. a mix).
+    """
+    # Do any extensions depend on too old lab packages?
+    any_older = False
+    # Do any extensions depend on too new lab packages?
+    any_newer = False
+
+    for _, jlab, ext in errors:
+        c = _compare_ranges(ext, jlab)
+        any_newer = any_newer or c < 0
+        any_older = any_older or c > 0
+    if any_older and not any_newer:
+        return 1
+    elif any_newer and not any_older:
+        return -1
+    return 0
+
+
 def _get_core_extensions():
     """Get the core extensions.
     """
